@@ -1,6 +1,6 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
-    Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
+    2012, 2013, 2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include <cctype>
 #include <cstdio>
 #include <vector>
+#include <stdint.h>
 
 #include "common.h"
 #include "rectangle.h"
@@ -48,8 +49,8 @@ Bitmap::Bitmap( const Bitmap & source, const Rectangle & re )
   for( int row = 0; row < height(); ++row )
     {
     data[row].resize( width() );
-    std::vector< unsigned char > & datarow = data[row];
-    const std::vector< unsigned char > & datarow2 = source.data[row+tdiff];
+    std::vector< uint8_t > & datarow = data[row];
+    const std::vector< uint8_t > & datarow2 = source.data[row+tdiff];
     for( int col = 0; col < width(); ++col )
       datarow[col] = datarow2[col+ldiff];
     }
@@ -74,7 +75,7 @@ void Bitmap::top( const int t )
   if( t == top() ) return;
   if( t < top() )
     data.insert( data.begin(), top() - t,
-                 std::vector< unsigned char >( width(), false ) );
+                 std::vector< uint8_t >( width(), false ) );
   else
     data.erase( data.begin(), data.begin() + ( t - top() ) );
   Rectangle::top( t );
@@ -171,7 +172,7 @@ bool Bitmap::adjust_width()
 
 // Return the total filled area of this Bitmap
 //
-int Bitmap::area() const throw()
+int Bitmap::area() const
   {
   int a = 0;
 
@@ -185,7 +186,7 @@ int Bitmap::area() const throw()
 
 // Return the central octagon filled area of this Bitmap
 //
-int Bitmap::area_octagon() const throw()
+int Bitmap::area_octagon() const
   {
   int a = 0;
   int bevel = ( 29 * std::min( height(), width() ) ) / 100;
@@ -210,14 +211,14 @@ int Bitmap::area_octagon() const throw()
 
 // Return the size of the central octagon of this blob
 //
-int Bitmap::size_octagon() const throw()
+int Bitmap::size_octagon() const
   {
   int bevel = ( 29 * std::min( height(), width() ) ) / 100;
   return size() - ( 2 * bevel * ( bevel + 1 ) );
   }
 
 
-int Bitmap::seek_left( const int row, const int col, const bool black ) const throw()
+int Bitmap::seek_left( const int row, const int col, const bool black ) const
   {
   int c = col;
   while( c > left() && get_bit( row, c - 1 ) != black ) --c;
@@ -225,7 +226,7 @@ int Bitmap::seek_left( const int row, const int col, const bool black ) const th
   }
 
 
-int Bitmap::seek_top( const int row, const int col, const bool black ) const throw()
+int Bitmap::seek_top( const int row, const int col, const bool black ) const
   {
   int r = row;
   while( r > top() && get_bit( r - 1, col ) != black ) --r;
@@ -233,7 +234,7 @@ int Bitmap::seek_top( const int row, const int col, const bool black ) const thr
   }
 
 
-int Bitmap::seek_right( const int row, const int col, const bool black ) const throw()
+int Bitmap::seek_right( const int row, const int col, const bool black ) const
   {
   int c = col;
   while( c < right() && get_bit( row, c + 1 ) != black ) ++c;
@@ -241,7 +242,7 @@ int Bitmap::seek_right( const int row, const int col, const bool black ) const t
   }
 
 
-int Bitmap::seek_bottom( const int row, const int col, const bool black ) const throw()
+int Bitmap::seek_bottom( const int row, const int col, const bool black ) const
   {
   int r = row;
   while( r < bottom() && get_bit( r + 1, col ) != black ) ++r;
@@ -249,7 +250,7 @@ int Bitmap::seek_bottom( const int row, const int col, const bool black ) const
   }
 
 
-bool Bitmap::escape_left( int row, int col ) const throw()
+bool Bitmap::escape_left( int row, int col ) const
   {
   if( get_bit( row, col ) ) return false;
   int u, d;
@@ -267,7 +268,7 @@ bool Bitmap::escape_left( int row, int col ) const throw()
   }
 
 
-bool Bitmap::escape_top( int row, int col ) const throw()
+bool Bitmap::escape_top( int row, int col ) const
   {
   if( get_bit( row, col ) ) return false;
   int l, r;
@@ -285,7 +286,7 @@ bool Bitmap::escape_top( int row, int col ) const throw()
   }
 
 
-bool Bitmap::escape_right( int row, int col ) const throw()
+bool Bitmap::escape_right( int row, int col ) const
   {
   if( get_bit( row, col ) ) return false;
   int u, d;
@@ -303,7 +304,7 @@ bool Bitmap::escape_right( int row, int col ) const throw()
   }
 
 
-bool Bitmap::escape_bottom( int row, int col ) const throw()
+bool Bitmap::escape_bottom( int row, int col ) const
   {
   if( get_bit( row, col ) ) return false;
   int l, r;
@@ -324,7 +325,7 @@ bool Bitmap::escape_bottom( int row, int col ) const throw()
 int Bitmap::follow_top( int row, int col ) const
   {
   if( !get_bit( row, col ) ) return row;
-  std::vector< unsigned char > array;
+  std::vector< uint8_t > array;
   array.reserve( width() );
   int c;
 
@@ -356,7 +357,7 @@ int Bitmap::follow_top( int row, int col ) const
 int Bitmap::follow_bottom( int row, int col ) const
   {
   if( !get_bit( row, col ) ) return row;
-  std::vector< unsigned char > array;
+  std::vector< uint8_t > array;
   array.reserve( width() );
   int c;
 
