@@ -1,6 +1,6 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
-    Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
+    2012, 2013, 2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 #include <algorithm>
 #include <cstdio>
 #include <vector>
+#include <stdint.h>
 
 #include "common.h"
 #include "rectangle.h"
@@ -31,7 +32,7 @@ namespace {
 void delete_hole( std::vector< Bitmap * > & holep_vector,
                   std::vector< Bitmap * > & v1,
                   std::vector< Bitmap * > & v2,
-                  Bitmap * const p, int i ) throw()
+                  Bitmap * const p, int i )
   {
   std::replace( v1.begin() + i, v1.end(), p, (Bitmap *) 0 );
   std::replace( v2.begin(), v2.begin() + i, p, (Bitmap *) 0 );
@@ -67,11 +68,11 @@ inline void join_holes( std::vector< Bitmap * > & holep_vector,
 
 
 void delete_outer_holes( const Rectangle & re,
-                         std::vector< Bitmap * > & holepv ) throw()
+                         std::vector< Bitmap * > & holepv )
   {
-  for( int i = holepv.size() - 1; i >= 0; --i )
+  for( unsigned i = holepv.size(); i > 0; )
     {
-    Bitmap & h = *holepv[i];
+    Bitmap & h = *holepv[--i];
     if( !re.strictly_includes( h ) )
       { delete &h; holepv.erase( holepv.begin() + i ); }
     }
@@ -83,7 +84,7 @@ void delete_outer_holes( const Rectangle & re,
 Blob::Blob( const Blob & b )
   : Bitmap( b ), holepv( b.holepv )
   {
-  for( unsigned int i = 0; i < holepv.size(); ++i )
+  for( unsigned i = 0; i < holepv.size(); ++i )
     holepv[i] = new Bitmap( *b.holepv[i] );
   }
 
@@ -93,18 +94,18 @@ Blob & Blob::operator=( const Blob & b )
   if( this != &b )
     {
     Bitmap::operator=( b );
-    for( unsigned int i = 0; i < holepv.size(); ++i ) delete holepv[i];
+    for( unsigned i = 0; i < holepv.size(); ++i ) delete holepv[i];
     holepv = b.holepv;
-    for( unsigned int i = 0; i < holepv.size(); ++i )
+    for( unsigned i = 0; i < holepv.size(); ++i )
       holepv[i] = new Bitmap( *b.holepv[i] );
     }
   return *this;
   }
 
 
-Blob::~Blob() throw()
+Blob::~Blob()
   {
-  for( unsigned int i = 0; i < holepv.size(); ++i ) delete holepv[i];
+  for( unsigned i = 0; i < holepv.size(); ++i ) delete holepv[i];
   }
 
 
@@ -136,7 +137,7 @@ void Blob::bottom( const int b )
   }
 
 
-const Bitmap & Blob::hole( const int i ) const throw()
+const Bitmap & Blob::hole( const int i ) const
   {
   if( i < 0 || i >= holes() )
     Ocrad::internal_error( "hole, index out of bounds" );
@@ -144,7 +145,7 @@ const Bitmap & Blob::hole( const int i ) const throw()
   }
 
 
-int Blob::id( const int row, const int col ) const throw()
+int Blob::id( const int row, const int col ) const
   {
   if( this->includes( row, col ) )
     {
@@ -157,14 +158,52 @@ int Blob::id( const int row, const int col ) const throw()
   }
 
 
-void Blob::print( FILE * const outfile ) const throw()
+bool Blob::test_BD() const
+  {
+  const int wlimit = std::min( height(), width() ) / 2;
+  int lb = wlimit, rt = wlimit;			// index of first dot found
+  for( int i = 0; i < wlimit; ++i )
+    if( id( bottom() - i, left() + i ) == 1 ||
+        id( bottom() - i, left() + i + 1 ) == 1 )
+      { lb = i; break; }
+  for( int i = 0; i < wlimit; ++i )
+    if( id( top() + i, right() - i ) == 1 )
+      { rt = i; break; }
+  return ( rt >= 2 && 3 * lb <= rt );
+  }
+
+
+bool Blob::test_Q() const
+  {
+  const int wlimit = std::min( height(), width() ) / 2;
+  int ltwmax = 0, rbwmax = 0;
+  int ltimin = wlimit, rbimin = wlimit;		// index of first dot found
+  for( int disp = 0; disp < width() / 4; ++disp )
+    {
+    int ltw = 0, rbw = 0;
+    for( int i = 0; i < wlimit; ++i )
+      {
+      if( id( top() + i, left() + disp + i ) == 1 )
+        { ++ltw; if( ltimin > i ) ltimin = i; }
+      if( id( bottom() - i, right() - disp - i ) == 1 )
+        { ++rbw; if( rbimin > i ) rbimin = i; }
+      }
+    if( ltwmax < ltw ) ltwmax = ltw;
+    if( rbwmax < rbw ) rbwmax = rbw;
+    }
+  return ( ( ltimin > rbimin || rbimin == 0 ) &&
+           ( 2 * ltwmax < rbwmax || ( 2 * ltwmax == rbwmax && rbwmax >= 4 ) ) );
+  }
+
+
+void Blob::print( FILE * const outfile ) const
   {
   for( int row = top(); row <= bottom(); ++row )
     {
     for( int col = left(); col <= right(); ++col )
       {
       if( get_bit( row, col ) ) std::fprintf( outfile, " O" );
-      else std::fprintf( outfile, " " );
+      else std::fprintf( outfile, " ." );
       }
     std::fputs( "\n", outfile );
     }
@@ -184,7 +223,7 @@ void Blob::fill_hole( const int i )
 
 void Blob::find_holes()
   {
-  for( unsigned int i = 0; i < holepv.size(); ++i ) delete holepv[i];
+  for( unsigned i = 0; i < holepv.size(); ++i ) delete holepv[i];
   holepv.clear();
   if( height() < 3 || width() < 3 ) return;
 
@@ -198,8 +237,8 @@ void Blob::find_holes()
     for( int col = left() + 1; col < right(); ++col )
       {
       const int dcol = col - left();
-      if( get_bit( row, col ) ) new_data[dcol] = this;	// black point
-      else						// white point
+      if( get_bit( row, col ) ) new_data[dcol] = this;	// black pixel
+      else						// white pixel
         {
         Bitmap *p;
         Bitmap *lp = new_data[dcol-1];
@@ -233,9 +272,9 @@ void Blob::find_holes()
       }
     }
 
-  for( int i = holepv.size() - 1; i >= 0; --i )	// FIXME noise holes removal
+  for( unsigned i = holepv.size(); i > 0; )	// FIXME noise holes removal
     {
-    Bitmap & h = *holepv[i];
+    Bitmap & h = *holepv[--i];
     if( this->strictly_includes( h ) &&
         ( h.height() > 4 || h.width() > 4 ||
           ( ( h.height() > 2 || h.width() > 2 ) && h.area() > 3 ) ) )
@@ -243,9 +282,9 @@ void Blob::find_holes()
     delete &h; holepv.erase( holepv.begin() + i );
     }
 /*
-  for( int i = holepv.size() - 1; i >= 0; --i )
+  for( unsigned i = holepv.size(); i > 0; )
     {
-    Bitmap & h = *holepv[i];
+    Bitmap & h = *holepv[--i];
     if( !this->strictly_includes( h ) )
       { delete &h; holepv.erase( holepv.begin() + i ); }
     if( 20 * h.height() < height() && 16 * h.width() < width() ) fill_hole( i );
