@@ -1,6 +1,6 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
-    Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
+    2012, 2013, 2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,16 +21,16 @@ class Blob : public Bitmap
   std::vector< Bitmap * > holepv;		// vector of holes
 
 public:
-  Blob( const int l, const int t, const int r, const int b ) throw()
+  Blob( const int l, const int t, const int r, const int b )
     : Bitmap( l, t, r, b ) {}
 
-  Blob( const Bitmap & source, const Rectangle & re ) throw()
+  Blob( const Bitmap & source, const Rectangle & re )
     : Bitmap( source, re ) {}
 
-  Blob( const Blob & b ) throw();
-  Blob & operator=( const Blob & b ) throw();
+  Blob( const Blob & b );
+  Blob & operator=( const Blob & b );
 
-  ~Blob() throw();
+  ~Blob();
 
   using Bitmap::left;
   using Bitmap::top;
@@ -38,20 +38,24 @@ public:
   using Bitmap::bottom;
   using Bitmap::height;
   using Bitmap::width;
-  void left  ( const int l ) throw();
-  void top   ( const int t ) throw();
-  void right ( const int r ) throw();
-  void bottom( const int b ) throw();
-  void height( const int h ) throw() { bottom( top() + h - 1 ); }
-  void width ( const int w ) throw() { right( left() + w - 1 ); }
+  void left  ( const int l );
+  void top   ( const int t );
+  void right ( const int r );
+  void bottom( const int b );
+  void height( const int h ) { bottom( top() + h - 1 ); }
+  void width ( const int w ) { right( left() + w - 1 ); }
 
-  const Bitmap & hole( const int i ) const throw();
-  int holes() const throw() { return holepv.size(); }
+  const Bitmap & hole( const int i ) const;
+  int holes() const { return holepv.size(); }
   //  id = 1 for blob dots, negative for hole dots, 0 otherwise
-  int id( const int row, const int col ) const throw();
+  int id( const int row, const int col ) const;
 
-  void print( FILE * outfile ) const throw();
+  bool is_abnormal() const
+    { return height() < 10 || height() >= 5 * width() || width() >= 3 * height(); }
+  bool test_BD() const;
+  bool test_Q() const;
+  void print( FILE * const outfile ) const;
 
-  void fill_hole( const int i ) throw();
-  void find_holes() throw();
+  void fill_hole( const int i );
+  void find_holes();
   };
