@@ -1,6 +1,6 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
-    Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
+    2012, 2013, 2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 #include <algorithm>
 #include <cstdio>
 #include <vector>
+#include <stdint.h>
 
 #include "common.h"
 #include "rectangle.h"
@@ -31,7 +32,7 @@
 Character::Character( const Character & c )
   : Rectangle( c ), blobpv( c.blobpv ), gv( c.gv )
   {
-  for( unsigned int i = 0; i < blobpv.size(); ++i )
+  for( unsigned i = 0; i < blobpv.size(); ++i )
     blobpv[i] = new Blob( *c.blobpv[i] );
   }
 
@@ -41,9 +42,9 @@ Character & Character::operator=( const Character & c )
   if( this != &c )
     {
     Rectangle::operator=( c );
-    for( unsigned int i = 0; i < blobpv.size(); ++i ) delete blobpv[i];
+    for( unsigned i = 0; i < blobpv.size(); ++i ) delete blobpv[i];
     blobpv = c.blobpv;
-    for( unsigned int i = 0; i < blobpv.size(); ++i )
+    for( unsigned i = 0; i < blobpv.size(); ++i )
       blobpv[i] = new Blob( *c.blobpv[i] );
     gv = c.gv;
     }
@@ -51,15 +52,15 @@ Character & Character::operator=( const Character & c )
   }
 
 
-Character::~Character() throw()
+Character::~Character()
   {
-  for( unsigned int i = 0; i < blobpv.size(); ++i ) delete blobpv[i];
+  for( unsigned i = 0; i < blobpv.size(); ++i ) delete blobpv[i];
   }
 
 
 // Return the filled area of the main blobs only (no recursive)
 //
-int Character::area() const throw()
+int Character::area() const
   {
   int a = 0;
   for( int i = 0; i < blobs(); ++i ) a += blobpv[i]->area();
@@ -67,7 +68,7 @@ int Character::area() const throw()
   }
 
 
-const Blob & Character::blob( const int i ) const throw()
+const Blob & Character::blob( const int i ) const
   {
   if( i < 0 || i >= blobs() )
     Ocrad::internal_error( "const blob, index out of bounds" );
@@ -75,7 +76,7 @@ const Blob & Character::blob( const int i ) const throw()
   }
 
 
-Blob & Character::blob( const int i ) throw()
+Blob & Character::blob( const int i )
   {
   if( i < 0 || i >= blobs() )
     Ocrad::internal_error( "blob, index out of bounds" );
@@ -83,7 +84,7 @@ Blob & Character::blob( const int i ) throw()
   }
 
 
-Blob & Character::main_blob() throw()
+Blob & Character::main_blob()
   {
   int imax = 0;
   for( int i = 1; i < blobs(); ++i )
@@ -115,7 +116,7 @@ void Character::insert_guess( const int i, const int code, const int value )
   }
 
 
-void Character::delete_guess( const int i ) throw()
+void Character::delete_guess( const int i )
   {
   if( i < 0 || i >= guesses() )
     Ocrad::internal_error( "delete_guess, index out of bounds" );
@@ -139,7 +140,7 @@ bool Character::set_merged_guess( const int code1, const int right1,
   }
 
 
-void Character::swap_guesses( const int i, const int j ) throw()
+void Character::swap_guesses( const int i, const int j )
   {
   if( i < 0 || i >= guesses() || j < 0 || j >= guesses() )
     Ocrad::internal_error( "swap_guesses, index out of bounds" );
@@ -148,7 +149,7 @@ void Character::swap_guesses( const int i, const int j ) throw()
   }
 
 
-const Character::Guess & Character::guess( const int i ) const throw()
+const Character::Guess & Character::guess( const int i ) const
   {
   if( i < 0 || i >= guesses() )
     Ocrad::internal_error( "guess, index out of bounds" );
@@ -156,7 +157,7 @@ const Character::Guess & Character::guess( const int i ) const throw()
   }
 
 
-bool Character::maybe( const int code ) const throw()
+bool Character::maybe( const int code ) const
   {
   for( int i = 0; i < guesses(); ++i )
     if( code == gv[i].code ) return true;
@@ -164,7 +165,7 @@ bool Character::maybe( const int code ) const throw()
   }
 
 /*
-bool Character::maybe_digit() const throw()
+bool Character::maybe_digit() const
   {
   for( int i = 0; i < guesses(); ++i )
     if( UCS::isdigit( gv[i].code ) ) return true;
@@ -172,7 +173,7 @@ bool Character::maybe_digit() const throw()
   }
 
 
-bool Character::maybe_letter() const throw()
+bool Character::maybe_letter() const
   {
   for( int i = 0; i < guesses(); ++i )
     if( UCS::isalpha( gv[i].code ) ) return true;
@@ -187,7 +188,7 @@ void Character::join( Character & c )
   }
 
 
-unsigned char Character::byte_result() const throw()
+unsigned char Character::byte_result() const
   {
   if( guesses() )
     {
@@ -198,44 +199,49 @@ unsigned char Character::byte_result() const throw()
   }
 
 
-void Character::print( const Control & control ) const throw()
+const char * Character::utf8_result() const
   {
   if( guesses() )
     {
-//    if( maybe( '\t' ) ) std::putc( '\t', control.outfile ); else
-    switch( control.format )
+    const char * s = UCS::ucs_to_utf8( gv[0].code );
+    if( *s ) return s;
+    }
+  return "_";
+  }
+
+
+void Character::print( const Control & control ) const
+  {
+  if( guesses() )
+    {
+    if( !control.utf8 )
       {
-      case Control::byte:
-        { unsigned char ch = UCS::map_to_byte( gv[0].code );
-        if( ch ) std::putc( ch, control.outfile ); }
-        break;
-      case Control::utf8:
-        if( gv[0].code )
-          std::fputs( UCS::ucs_to_utf8( gv[0].code ), control.outfile );
+      unsigned char ch = UCS::map_to_byte( gv[0].code );
+      if( ch ) std::putc( ch, control.outfile );
       }
+    else if( gv[0].code )
+      std::fputs( UCS::ucs_to_utf8( gv[0].code ), control.outfile );
     }
   else std::putc( '_', control.outfile );
   }
 
 
 void Character::dprint( const Control & control, const Rectangle & charbox,
-                        const bool graph, const bool recursive ) const throw()
+                        const bool graph, const bool recursive ) const
   {
   if( graph || recursive )
     std::fprintf( control.outfile, "%d guesses    ", guesses() );
   for( int i = 0; i < guesses(); ++i )
     {
-    switch( control.format )
+    if( !control.utf8 )
       {
-      case Control::byte:
-        { unsigned char ch = UCS::map_to_byte( gv[i].code );
-        if( ch ) std::fprintf( control.outfile, "guess '%c', confidence %d    ",
-                               ch, gv[i].value ); }
-        break;
-      case Control::utf8:
-        std::fprintf( control.outfile, "guess '%s', confidence %d    ",
-                      UCS::ucs_to_utf8( gv[i].code ), gv[i].value );
+      unsigned char ch = UCS::map_to_byte( gv[i].code );
+      if( ch ) std::fprintf( control.outfile, "guess '%c', confidence %d    ",
+                             ch, gv[i].value );
       }
+    else
+      std::fprintf( control.outfile, "guess '%s', confidence %d    ",
+                    UCS::ucs_to_utf8( gv[i].code ), gv[i].value );
     if( !graph && !recursive ) break;
     }
   std::fputs( "\n", control.outfile );
@@ -274,7 +280,7 @@ void Character::dprint( const Control & control, const Rectangle & charbox,
 
       for( int col = left(); col <= right(); ++col )
         {
-        char ch = ( isvc && col == hcenter() ) ? '+' : '';
+        char ch = ( isvc && col == hcenter() ) ? '+' : '.';
         for( int i = 0; i < blobs(); ++i )
           {
           int id = blobpv[i]->id( row, col );
@@ -307,31 +313,29 @@ void Character::dprint( const Control & control, const Rectangle & charbox,
   }
 
 
-void Character::xprint( const Control & control ) const throw()
+void Character::xprint( const Control & control ) const
   {
   std::fprintf( control.exportfile, "%3d %3d %2d %2d; %d",
                 left(), top(), width(), height(), guesses() );
 
   for( int i = 0; i < guesses(); ++i )
-    switch( control.format )
+    if( !control.utf8 )
       {
-      case Control::byte:
-        { unsigned char ch = UCS::map_to_byte( gv[i].code );
-        if( !ch ) ch = '_';
-        std::fprintf( control.exportfile, ", '%c'%d", ch, gv[i].value ); }
-        break;
-      case Control::utf8:
-        std::fprintf( control.exportfile, ", '%s'%d",
-                      UCS::ucs_to_utf8( gv[i].code ), gv[i].value );
+      unsigned char ch = UCS::map_to_byte( gv[i].code );
+      if( !ch ) ch = '_';
+      std::fprintf( control.exportfile, ", '%c'%d", ch, gv[i].value );
       }
+    else
+      std::fprintf( control.exportfile, ", '%s'%d",
+                    UCS::ucs_to_utf8( gv[i].code ), gv[i].value );
   std::fputs( "\n", control.exportfile );
   }
 
 
 void Character::apply_filter( const Filter & filter )
   {
-  if( filter.type() == Filter::none ) return;
-  const int code = guesses() ? gv[0].code : 0;
+  if( filter.type() == Filter::none || !guesses() ) return;
+  const int code = gv[0].code;
   bool remove = false;
 
   switch( filter.type() )
@@ -345,10 +349,9 @@ void Character::apply_filter( const Filter & filter )
         {
         for( int i = 1; i < guesses(); ++i )
           if( UCS::isalpha( gv[i].code ) ) { swap_guesses( 0, i ); break; }
-        if( guesses() && !UCS::isalpha( gv[0].code ) )
+        if( !UCS::isalpha( gv[0].code ) )
           gv[0].code = UCS::to_nearest_letter( gv[0].code );
-        if( remove && ( !guesses() || !UCS::isalpha( gv[0].code ) ) )
-          only_guess( 0, 0 );
+        if( remove && !UCS::isalpha( gv[0].code ) ) clear_guesses();
         }
       break;
     case Filter::numbers_only:
@@ -358,10 +361,9 @@ void Character::apply_filter( const Filter & filter )
         {
         for( int i = 1; i < guesses(); ++i )
           if( UCS::isdigit( gv[i].code ) ) { swap_guesses( 0, i ); break; }
-        if( guesses() && !UCS::isdigit( gv[0].code ) )
+        if( !UCS::isdigit( gv[0].code ) )
           gv[0].code = UCS::to_nearest_digit( gv[0].code );
-        if( remove && ( !guesses() || !UCS::isdigit( gv[0].code ) ) )
-          only_guess( 0, 0 );
+        if( remove && !UCS::isdigit( gv[0].code ) ) clear_guesses();
         }
       break;
     }
