@@ -1,6 +1,6 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
-    Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
+    2012, 2013, 2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,9 +19,11 @@
 #include <cstdio>
 #include <cstdlib>
 #include <vector>
+#include <stdint.h>
 
 #include "common.h"
 #include "rectangle.h"
+#include "segment.h"
 #include "ucs.h"
 #include "bitmap.h"
 #include "blob.h"
@@ -33,7 +35,7 @@
 // Recognizes 2 blob characters.
 // ij!%:;=?|
 //
-void Character::recognize12( const Charset & charset, const Rectangle & charbox ) throw()
+void Character::recognize12( const Charset & charset, const Rectangle & charbox )
   {
   const Blob & b1 = blob( 0 );		// upper blob
   const Blob & b2 = blob( 1 );		// lower blob
@@ -74,20 +76,18 @@ void Character::recognize12( const Charset & charset, const Rectangle & charbox
         { add_guess( UCS::PLUSMIN, 0 ); return; }
       }
     if( b1.includes_hcenter( b2 ) && b2.includes_hcenter( b1 ) )
-      { if( b1.holes() && b2.holes() ) { add_guess( 'g', 0 ); return; } }
-    if( b1.hcenter() < b2.hcenter() )
       {
-      // Looks for merged 'fi'
+      if( b1.holes() && b2.holes() ) { add_guess( 'g', 0 ); return; }
+      }
+    if( b1.hcenter() < b2.hcenter() )		// Looks for merged 'fi'
+      {
       if( b2.height() > b2.width() && b1.hcenter() < b2.left() &&
           b1.includes_hcenter( b2 ) && 4 * b1.height() > 5 * b2.height() &&
           Ocrad::similar( b1.bottom()-b1.top(), b2.bottom()-b1.top(), 10 ) )
         {
         Character c2( new Blob( b2 ) ); c2.recognize1( charset, charbox );
-        if( c2.maybe('l') || c2.maybe('|') )
-          {
-          only_guess( 0, b1.left() ); add_guess( 'f', b2.left() );
-          add_guess( 'i', b2.right() ); return;
-          }
+        if( ( c2.maybe('l') || c2.maybe('|') ) &&
+            set_merged_guess( 'f', b2.left() - 1, 'i', 0 ) ) return;
         }
       }
     }
@@ -104,7 +104,7 @@ void Character::recognize12( const Charset & charset, const Rectangle & charbox
       int code = f1.test_solid( charbox );		//FIXME all this
       if( code == '-' && 2 * b1.height() > b1.width() ) code = '.';
       else if( code == '\'' || code == '|' ) code = '.';
-      if( !code && !b1.holes() && 
+      if( !code && !b1.holes() &&
           2 * b1.height() < b2.height() && b1.width() <= b2.width() )
         {
         if( 10 * a1 >= 7 * b1.height() * b1.width() ) code = '.';
@@ -126,14 +126,12 @@ void Character::recognize12( const Charset & charset, const Rectangle & charbox
               if( b2.seek_bottom( b2.vcenter(), col ) < b2.bottom() ) break;
             if( b2.left() < col && col < b2.right() )
               {
-              only_guess( 0, b2.left() );
               if( charset.enabled( Charset::iso_8859_9 ) && f2.rp.istip() )
-                { add_guess( 'T', col - 1 ); add_guess( UCS::CIDOT, b2.right() ); }
+                set_merged_guess( 'T', col - 1, UCS::CIDOT, 1 );
               else
                 {
-                add_guess( 'r', col - 1 );
-                if( code == '.' ) add_guess( 'i', b2.right() );
-                else add_guess( UCS::SIACUTE, b2.right() );
+                const int code2 = ( ( code == '.' ) ? 'i' : (int)UCS::SIACUTE );
+                set_merged_guess( 'r', col - 1, code2, 1 );
                 }
               return;
               }
@@ -231,7 +229,7 @@ void Character::recognize12( const Charset & charset, const Rectangle & charbox
   if( b1.bottom() <= b2.top() )
     {
     const int code = f2.test_solid( charbox );
-    if( !b1.holes() && ( code == '.' || 
+    if( !b1.holes() && ( code == '.' ||
           ( code && Ocrad::similar( b2.height(), b2.width(), 50 ) ) ) )
       {
       if( Ocrad::similar( b1.width(), b2.width(), 50 ) && !f1.lp.isctip() )
