@@ -1,6 +1,6 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
-    Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
+    2012, 2013, 2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,9 +21,9 @@ extern int verbosity;
 
 namespace Ocrad {
 
-void internal_error( const char * const msg ) throw();
+void internal_error( const char * const msg );
 bool similar( const int a, const int b,
-              const int percent_dif, const int abs_dif = 1 ) throw();
+              const int percent_dif, const int abs_dif = 1 );
 
 } // end namespace Ocrad
 
@@ -35,12 +35,12 @@ class Charset
 public:
   enum Value { ascii = 1, iso_8859_9 = 2, iso_8859_15 = 4 };
 
-  Charset() throw() : charset_( 0 ) {}
-  bool enable( const char * const name ) throw();
-  bool enabled( const Value cset ) const throw();
-  bool only( const Value cset ) const throw();
+  Charset() : charset_( 0 ) {}
+  bool enable( const char * const name );
+  bool enabled( const Value cset ) const;
+  bool only( const Value cset ) const;
   void show_error( const char * const program_name,
-                   const char * const arg ) const throw();
+                   const char * const arg ) const;
   };
 
 
@@ -52,11 +52,11 @@ private:
   Type type_;
 
 public:
-  Filter() throw() : type_( none ) {}
-  bool set( const char * const name ) throw();
-  Type type() const throw() { return type_; }
+  Filter() : type_( none ) {}
+  bool set( const char * const name );
+  Type type() const { return type_; }
   void show_error( const char * const program_name,
-                   const char * const arg ) const throw();
+                   const char * const arg ) const;
   };
 
 
@@ -69,28 +69,26 @@ private:
   Type type_;
 
 public:
-  Transformation() throw() : type_( none ) {}
-  bool set( const char * const name ) throw();
-  Type type() const throw() { return type_; }
+  Transformation() : type_( none ) {}
+  bool set( const char * const name );
+  Type type() const { return type_; }
   void show_error( const char * const program_name,
-                   const char * const arg ) const throw();
+                   const char * const arg ) const;
   };
 
 
 struct Control
   {
-  enum Format { byte, utf8 };
-
   Charset charset;
   Filter filter;
-  Format format;
   FILE * outfile, * exportfile;
   int debug_level;
   char filetype;
+  bool utf8;
 
-  Control() throw()
-    : format( byte ), outfile( stdout ), exportfile( 0 ),
-      debug_level( 0 ), filetype( '4' ) {}
+  Control()
+    : outfile( stdout ), exportfile( 0 ),
+      debug_level( 0 ), filetype( '4' ), utf8( false ) {}
 
-  bool set_format( const char * const name ) throw();
+  bool set_format( const char * const name );
   };
