@@ -1,6 +1,6 @@
 /*  GNU Ocrad - Optical Character Recognition program
     Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
-    2012, 2013 Antonio Diaz Diaz.
+    2012, 2013, 2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 
 #include <algorithm>
 #include <cstdio>
+#include <cstdlib>
 #include <vector>
 #include <stdint.h>
 
@@ -171,7 +172,10 @@ int Features::test_CEFIJLlT( const Charset & charset ) const
       if( hdiff > b.height() / 2 && rp.increasing( rp.pos( 80 ), 1 ) &&
           !rp.decreasing() ) return 'J';
       if( -hdiff > b.height() / 2 )
-        { if( 5 * lp[lp.pos(80)] < 2 * b.width() ) return 'L'; else return 'v'; }
+        {
+        if( 5 * lp[lp.pos(80)] >= 2 * b.width() ) return 'v';
+        if( col > b.hcenter() ) return 'L';
+        }
       }
     }
   }
@@ -831,9 +835,10 @@ int Features::test_solid( const Rectangle & charbox ) const
 
   if( b.width() >= 3 && b.height() >= 3 )
     {
+    const int vnoise = ( b.height() / 100 ) + 1;
     inner_size = ( b.width() - 2 ) * ( b.height() - 2 );
     inner_area = 0;
-    for( int row = b.top() + 1; row < b.bottom(); ++row )
+    for( int row = b.top() + vnoise; row <= b.bottom() - vnoise; ++row )
       {
       int holes = 0;	// FIXME
       for( int col = b.left() + 1; col < b.right(); ++col )
