@@ -1,5 +1,5 @@
 /*  Ocradlib - Optical Character Recognition library
-    Copyright (C) 2009, 2010 Antonio Diaz Diaz.
+    Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Antonio Diaz Diaz.
 
     This library is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -12,24 +12,14 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-
-    As a special exception, you may use this file as part of a free
-    software library without restriction.  Specifically, if other files
-    instantiate templates or use macros or inline functions from this
-    file, or you compile this file and link it with other files to
-    produce an executable, this file does not by itself cause the
-    resulting executable to be covered by the GNU General Public
-    License.  This exception does not however invalidate any other
-    reasons why the executable file might be covered by the GNU General
-    Public License.
+    along with this library.  If not, see <http://www.gnu.org/licenses/>.
 */
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-const char * const OCRAD_version_string = "0.20";
+const char * const OCRAD_version_string = "0.23";
 
 
 /* OCRAD_Pixmap.data is a pointer to image data formed by "height" rows
@@ -69,19 +59,22 @@ enum OCRAD_Errno OCRAD_get_errno( struct OCRAD_Descriptor * const ocrdes );
 
 int OCRAD_set_image( struct OCRAD_Descriptor * const ocrdes,
                      const struct OCRAD_Pixmap * const image,
-                     const unsigned char invert );	// true/false
+                     const bool invert );
 
 int OCRAD_set_image_from_file( struct OCRAD_Descriptor * const ocrdes,
                                const char * const filename,
-                               const unsigned char invert );	// true/false
+                               const bool invert );
+
+int OCRAD_set_utf8_format( struct OCRAD_Descriptor * const ocrdes,
+                           const bool utf8 );		// 0 = byte, 1 = utf8
 
 int OCRAD_set_threshold( struct OCRAD_Descriptor * const ocrdes,
                          const int threshold );		// 0..255, -1 = auto
 
-int OCRAD_scale( OCRAD_Descriptor * const ocrdes, const int value );
+int OCRAD_scale( struct OCRAD_Descriptor * const ocrdes, const int value );
 
 int OCRAD_recognize( struct OCRAD_Descriptor * const ocrdes,
-                     const unsigned char layout );	// true/false
+                     const bool layout );
 
 int OCRAD_result_blocks( struct OCRAD_Descriptor * const ocrdes );
 
