@@ -1,11 +1,13 @@
 /*  Ocrcheck - A test program for the ocradlib library
-    Copyright (C) 2009, 2010 Antonio Diaz Diaz.
+    Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Antonio Diaz Diaz.
 
     This program is free software: you have unlimited permission
     to copy, distribute and modify it.
 
     Usage is:
       ocrcheck filename.pnm
+    or
+      ocrcheck filename.pnm --utf8
 
     This program reads the specified image file, feeds it to the OCR
     engine and sends the resulting text to stdout.
@@ -22,6 +24,7 @@
 
 int main( const int argc, const char * const argv[] )
   {
+  const bool utf8 = ( argc > 2 );
   if( argc < 2 )
     {
     std::fprintf( stderr, "Usage: ocrcheck filename.pnm\n" );
@@ -55,12 +58,13 @@ int main( const int argc, const char * const argv[] )
     if( ocr_errno == OCRAD_mem_error )
       std::fprintf( stderr, "not enough memory.\n" );
     else
-      std::fprintf( stderr, "Can't open file `%s' for reading\n", argv[1] );
+      std::fprintf( stderr, "Can't open file '%s' for reading\n", argv[1] );
     return 1;
     }
-//  std::fprintf( stderr, "ocrcheck: testing file `%s'\n", argv[1] );
+//  std::fprintf( stderr, "ocrcheck: testing file '%s'\n", argv[1] );
 
-  if( OCRAD_set_threshold( ocrdes, -1 ) < 0 ||	// auto threshold
+  if( ( utf8 && OCRAD_set_utf8_format( ocrdes, true ) < 0 ) ||
+      OCRAD_set_threshold( ocrdes, -1 ) < 0 ||	// auto threshold
       OCRAD_recognize( ocrdes, false ) < 0 )	// no layout
     {
     const OCRAD_Errno ocr_errno = OCRAD_get_errno( ocrdes );
@@ -89,7 +93,13 @@ int main( const int argc, const char * const argv[] )
       if( s && s[0] )
         {
         std::printf( "%s", s );
-        chars_total_by_count += std::strlen( s ) - 1;
+        const int len = std::strlen( s ) - 1;
+        if( !utf8 )
+          chars_total_by_count += len;
+        else
+          for( int i = 0; i < len; ++i )
+            if( (uint8_t)s[i] < 128 || (uint8_t)s[i] >= 0xC0 )
+              ++chars_total_by_count;
         }
       }
     std::printf( "\n" );
@@ -99,7 +109,9 @@ int main( const int argc, const char * const argv[] )
       chars_total_by_line != chars_total ||
       chars_total_by_count != chars_total )
     {
-    std::fprintf( stderr, "library_error: character counts differ.\n" );
+    std::fprintf( stderr, "library_error: character counts differ.\n"
+                  "%d  %d  %d  %d\n", chars_total, chars_total_by_block,
+                  chars_total_by_line, chars_total_by_count );
     return 1;
     }
 
