@@ -1,6 +1,6 @@
 /*  GNU Ocrad - Optical Character Recognition program
     Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
-    2012, 2013 Antonio Diaz Diaz.
+    2012, 2013, 2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include <cctype>
 #include <climits>
 #include <cstdio>
+#include <cstdlib>
 #include <string>
 #include <vector>
 #include <stdint.h>
@@ -295,7 +296,8 @@ void mirror_left_right( std::vector< std::vector< uint8_t > > & data )
 
 void mirror_top_bottom( std::vector< std::vector< uint8_t > > & data )
   {
-  for( int u = 0, d = data.size() - 1; u < d; ++u, --d )
+  const int height = data.size();
+  for( int u = 0, d = height - 1; u < d; ++u, --d )
     data[u].swap( data[d] );
   }
 
