@@ -1,6 +1,6 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
-    Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
+    2012, 2013, 2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,7 +18,9 @@
 
 #include <algorithm>
 #include <cstdio>
+#include <cstdlib>
 #include <vector>
+#include <stdint.h>
 
 #include "common.h"
 #include "rectangle.h"
@@ -27,7 +29,7 @@
 #include "profile.h"
 
 
-Profile::Profile( const Bitmap & bm_, const Type t ) throw()
+Profile::Profile( const Bitmap & bm_, const Type t )
   : bm( bm_ ), type( t ),
     limit_( -1 ), max_( -1 ), min_( -1 ), mean_( -1 ),
     isconcave_( -1 ), isconvex_( -1 ), isflat_( -1 ), isflats_( -1 ),
@@ -168,10 +170,10 @@ int Profile::area( const int l, int r )
   }
 
 
-bool Profile::increasing( int i )
+bool Profile::increasing( int i, const int min_delta )
   {
   if( limit_ < 0 ) initialize();
-  if( i < 0 || i > samples() - 2 || data[samples()-1] - data[i] < 2 )
+  if( i < 0 || i > samples() - 2 || data[samples()-1] - data[i] < min_delta )
     return false;
   while( ++i < samples() ) if( data[i] < data[i-1] ) return false;
   return true;
@@ -271,7 +273,7 @@ bool Profile::isflat()
   if( isflat_ < 0 )
     {
     isflat_ = false; if( limit_ < 0 ) initialize();
-    if( samples() < 15 ) return isflat_;
+    if( samples() < 10 ) return isflat_;
     int mn = data[samples()/2], mx = mn;
 
     for( int i = 1; i < samples() - 1; ++i )
@@ -287,7 +289,7 @@ bool Profile::isflats()
   if( isflats_ < 0 )
     {
     isflats_ = false; if( limit_ < 0 ) initialize();
-    if( samples() < 15 ) return isflats_;
+    if( samples() < 12 ) return isflats_;
     const int s1 = std::max( pos( 15 ), 3 );
     const int s2 = std::min( pos( 85 ), samples() - 4 );
     int mn = -1, mx = 0;
