@@ -1,6 +1,6 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
-    Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
+    2012, 2013, 2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -27,13 +27,13 @@
 
 namespace {
 
-void error( const char * const msg ) throw()
+void error( const char * const msg )
   { Ocrad::internal_error( msg ); }
 
 } // end namespace
 
 
-Rectangle::Rectangle( const int l, const int t, const int r, const int b ) throw()
+Rectangle::Rectangle( const int l, const int t, const int r, const int b )
   {
   if( r < l || b < t )
     {
@@ -45,56 +45,56 @@ Rectangle::Rectangle( const int l, const int t, const int r, const int b ) throw
   }
 
 
-void Rectangle::left( const int l ) throw()
+void Rectangle::left( const int l )
   {
   if( l > right_ ) error( "left, bad parameter resizing a Rectangle" );
   left_ = l;
   }
 
 
-void Rectangle::top( const int t ) throw()
+void Rectangle::top( const int t )
   {
   if( t > bottom_ ) error( "top, bad parameter resizing a Rectangle" );
   top_ = t;
   }
 
 
-void Rectangle::right( const int r ) throw()
+void Rectangle::right( const int r )
   {
   if( r < left_ ) error( "right, bad parameter resizing a Rectangle" );
   right_ = r;
   }
 
 
-void Rectangle::bottom( const int b ) throw()
+void Rectangle::bottom( const int b )
   {
   if( b < top_ ) error( "bottom, bad parameter resizing a Rectangle" );
   bottom_ = b;
   }
 
 
-void Rectangle::height( const int h ) throw()
+void Rectangle::height( const int h )
   {
   if( h <= 0 ) error( "height, bad parameter resizing a Rectangle" );
   bottom_ = top_ + h - 1;
   }
 
 
-void Rectangle::width( const int w ) throw()
+void Rectangle::width( const int w )
   {
   if( w <= 0 ) error( "width, bad parameter resizing a Rectangle" );
   right_ = left_ + w - 1;
   }
 
 
-void Rectangle::add_point( const int row, const int col ) throw()
+void Rectangle::add_point( const int row, const int col )
   {
   if( row > bottom_ ) bottom_ = row; else if( row < top_ ) top_ = row;
   if( col > right_ ) right_ = col;   else if( col < left_ ) left_ = col;
   }
 
 
-void Rectangle::add_rectangle( const Rectangle & re ) throw()
+void Rectangle::add_rectangle( const Rectangle & re )
   {
   if( re.left_ < left_ )     left_ = re.left_;
   if( re.top_ < top_ )       top_ = re.top_;
@@ -103,79 +103,79 @@ void Rectangle::add_rectangle( const Rectangle & re ) throw()
   }
 
 
-void Rectangle::enlarge( const int scale ) throw()
+void Rectangle::enlarge( const int scale )
   {
   if( scale > 1 )
     { left_ *= scale; top_ *= scale; right_ *= scale; bottom_ *= scale; }
   }
 
 
-void Rectangle::move( const int row, const int col ) throw()
+void Rectangle::move( const int row, const int col )
   {
   int d = row - top_; if( d ) { top_ += d; bottom_ += d; }
   d = col - left_; if( d ) { left_ += d; right_ += d; }
   }
 
 
-bool Rectangle::includes( const Rectangle & re ) const throw()
+bool Rectangle::includes( const Rectangle & re ) const
   {
   return ( left_  <= re.left_  && top_    <= re.top_ &&
            right_ >= re.right_ && bottom_ >= re.bottom_ );
   }
 
 
-bool Rectangle::includes( const int row, const int col ) const throw()
+bool Rectangle::includes( const int row, const int col ) const
   {
   return ( left_ <= col && right_ >= col && top_ <= row && bottom_ >= row );
   }
 
 
-bool Rectangle::strictly_includes( const Rectangle & re ) const throw()
+bool Rectangle::strictly_includes( const Rectangle & re ) const
   {
   return ( left_  < re.left_  && top_    < re.top_ &&
            right_ > re.right_ && bottom_ > re.bottom_ );
   }
 
 
-bool Rectangle::strictly_includes( const int row, const int col ) const throw()
+bool Rectangle::strictly_includes( const int row, const int col ) const
   {
   return ( left_ < col && right_ > col && top_ < row && bottom_ > row );
   }
 
 
-bool Rectangle::includes_hcenter( const Rectangle & re ) const throw()
+bool Rectangle::includes_hcenter( const Rectangle & re ) const
   { return ( left_ <= re.hcenter() && right_ >= re.hcenter() ); }
 
 
-bool Rectangle::includes_vcenter( const Rectangle & re ) const throw()
+bool Rectangle::includes_vcenter( const Rectangle & re ) const
   { return ( top_ <= re.vcenter() && bottom_ >= re.vcenter() ); }
 
 
-bool Rectangle::h_includes( const Rectangle & re ) const throw()
+bool Rectangle::h_includes( const Rectangle & re ) const
   { return ( left_ <= re.left_ && right_ >= re.right_ ); }
 
 
-bool Rectangle::v_includes( const Rectangle & re ) const throw()
+bool Rectangle::v_includes( const Rectangle & re ) const
   { return ( top_ <= re.top_ && bottom_ >= re.bottom_ ); }
 
 
-bool Rectangle::h_includes( const int col ) const throw()
+bool Rectangle::h_includes( const int col ) const
   { return ( left_ <= col && right_ >= col ); }
 
 
-bool Rectangle::v_includes( const int row ) const throw()
+bool Rectangle::v_includes( const int row ) const
   { return ( top_ <= row && bottom_ >= row ); }
 
 
-bool Rectangle::h_overlaps( const Rectangle & re ) const throw()
+bool Rectangle::h_overlaps( const Rectangle & re ) const
   { return ( left_ <= re.right_ && right_ >= re.left_ ); }
 
 
-bool Rectangle::v_overlaps( const Rectangle & re ) const throw()
+bool Rectangle::v_overlaps( const Rectangle & re ) const
   { return ( top_ <= re.bottom_ && bottom_ >= re.top_ ); }
 
 
-int Rectangle::v_overlap_percent( const Rectangle & re ) const throw()
+int Rectangle::v_overlap_percent( const Rectangle & re ) const
   {
   int ov = std::min( bottom_, re.bottom_ ) - std::max( top_, re.top_ ) + 1;
   if( ov > 0 )
@@ -185,7 +185,7 @@ int Rectangle::v_overlap_percent( const Rectangle & re ) const throw()
   }
 
 
-bool Rectangle::is_hcentred_in( const Rectangle & re ) const throw()
+bool Rectangle::is_hcentred_in( const Rectangle & re ) const
   {
   if( this->h_includes( re.hcenter() ) ) return true;
   int w = std::min( re.height(), re.width() ) / 2;
@@ -199,7 +199,7 @@ bool Rectangle::is_hcentred_in( const Rectangle & re ) const throw()
   }
 
 
-bool Rectangle::is_vcentred_in( const Rectangle & re ) const throw()
+bool Rectangle::is_vcentred_in( const Rectangle & re ) const
   {
   if( this->v_includes( re.vcenter() ) ) return true;
   int h = std::min( re.height(), re.width() ) / 2;
@@ -213,7 +213,7 @@ bool Rectangle::is_vcentred_in( const Rectangle & re ) const throw()
   }
 
 
-bool Rectangle::precedes( const Rectangle & re ) const throw()
+bool Rectangle::precedes( const Rectangle & re ) const
   {
   if( right_ < re.left_ ) return true;
   if( this->h_overlaps( re ) &&
@@ -223,11 +223,11 @@ bool Rectangle::precedes( const Rectangle & re ) const throw()
   }
 
 
-bool Rectangle::h_precedes( const Rectangle & re ) const throw()
+bool Rectangle::h_precedes( const Rectangle & re ) const
   { return ( hcenter() < re.hcenter() ); }
 
 
-bool Rectangle::v_precedes( const Rectangle & re ) const throw()
+bool Rectangle::v_precedes( const Rectangle & re ) const
   {
   if( bottom_ < re.vcenter() || vcenter() < re.top_ ) return true;
   if( this->includes_vcenter( re ) && re.includes_vcenter( *this ) )
@@ -236,36 +236,36 @@ bool Rectangle::v_precedes( const Rectangle & re ) const throw()
   }
 
 
-int Rectangle::distance( const Rectangle & re ) const throw()
+int Rectangle::distance( const Rectangle & re ) const
   { return hypoti( h_distance( re ), v_distance( re ) ); }
 
 
-int Rectangle::distance( const int row, const int col ) const throw()
+int Rectangle::distance( const int row, const int col ) const
   { return hypoti( h_distance( col ), v_distance( row ) ); }
 
 
-int Rectangle::h_distance( const Rectangle & re ) const throw()
+int Rectangle::h_distance( const Rectangle & re ) const
   {
   if( re.right_ <= left_ ) return left_ - re.right_;
   if( re.left_ >= right_ ) return re.left_ - right_;
   return 0;
   }
 
-int Rectangle::h_distance( const int col ) const throw()
+int Rectangle::h_distance( const int col ) const
   {
   if( col <= left_ ) return left_ - col;
   if( col >= right_ ) return col - right_;
   return 0;
   }
 
-int Rectangle::v_distance( const Rectangle & re ) const throw()
+int Rectangle::v_distance( const Rectangle & re ) const
   {
   if( re.bottom_ <= top_ ) return top_ - re.bottom_;
   if( re.top_ >= bottom_ ) return re.top_ - bottom_;
   return 0;
   }
 
-int Rectangle::v_distance( const int row ) const throw()
+int Rectangle::v_distance( const int row ) const
   {
   if( row <= top_ ) return top_ - row;
   if( row >= bottom_ ) return row - bottom_;
@@ -273,7 +273,7 @@ int Rectangle::v_distance( const int row ) const throw()
   }
 
 
-int Rectangle::hypoti( const int c1, const int c2 ) throw()
+int Rectangle::hypoti( const int c1, const int c2 )
   {
   long long temp = c1; temp *= temp;
   long long target = c2; target *= target; target += temp;
