@@ -1,6 +1,6 @@
 /*  GNU Ocrad - Optical Character Recognition program
     Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
-    2012, 2013 Antonio Diaz Diaz.
+    2012, 2013, 2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
