@@ -1,6 +1,6 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010
-    Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
+    2012, 2013, 2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,19 +20,22 @@ class Textblock : public Rectangle
   {
   mutable std::vector< Textline * > tlpv;
 
+  Textblock( const Textblock & );		// declared as private
+  void operator=( const Textblock & );		// declared as private
+
 public:
   Textblock( const Rectangle & page, const Rectangle & block,
-             std::vector< Blob * > & blobp_vector ) throw();
-  ~Textblock() throw();
-  void recognize( const Charset & charset, const Filter & filter ) throw();
+             std::vector< Blob * > & blobp_vector );
+  ~Textblock();
+  void recognize( const Charset & charset, const Filter & filter );
 
-  const Textline & textline( int i ) const throw();
-  int textlines() const throw() { return tlpv.size(); }
-  int characters() const throw();
+  const Textline & textline( const int i ) const;
+  int textlines() const { return tlpv.size(); }
+  int characters() const;
 
-  void print( const Control & control ) const throw();
-  void dprint( const Control & control, bool graph, bool recursive ) const throw();
-  void xprint( const Control & control ) const throw();
-  void cmark( Page_image & page_image ) const throw();
-  void lmark( Page_image & page_image ) const throw();
+  void print( const Control & control ) const;
+  void dprint( const Control & control, bool graph, bool recursive ) const;
+  void xprint( const Control & control ) const;
+  void cmark( Page_image & page_image ) const;
+  void lmark( Page_image & page_image ) const;
   };
