@@ -1,6 +1,6 @@
 /*  GNU Ocrad - Optical Character Recognition program
-    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011
-    Antonio Diaz Diaz.
+    Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
+    2012, 2013, 2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,10 +16,12 @@
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 
+#include <algorithm>
 #include <cctype>
 #include <cstdio>
 #include <cstdlib>
 #include <vector>
+#include <stdint.h>
 
 #include "common.h"
 #include "rectangle.h"
@@ -35,8 +37,7 @@
 
 namespace {
 
-int find_space_or_hyphen( const std::vector< Character * > & cpv,
-                          unsigned int i ) throw()
+int find_space_or_hyphen( const std::vector< Character * > & cpv, unsigned i )
   {
   while( i < cpv.size() && !cpv[i]->maybe(' ') && !cpv[i]->maybe('-') ) ++i;
   return i;
@@ -45,6 +46,61 @@ int find_space_or_hyphen( const std::vector< Character * > & cpv,
 } // end namespace
 
 
+     // transform some small letters to capitals
+void Textline::check_lower_ambiguous()
+  {
+  int begin = big_initials();
+  bool isolated = false;	// isolated letters compare with all line
+  for( int i = big_initials(); i < characters(); ++i )
+    {
+    Character & c1 = character( i );
+    if( c1.maybe(' ') )
+      {
+      if( i + 2 < characters() && character( i + 2 ).maybe(' ') )
+        { begin = big_initials(); isolated = true; }
+      else { begin = i + 1; isolated = false; }
+      continue;
+      }
+    if( c1.guesses() == 1 )
+      {
+      const int code = c1.guess( 0 ).code;
+      if( !UCS::islower_small_ambiguous( code ) ) continue;
+      if( 5 * c1.height() < 4 * mean_height() ) continue;
+      bool capital = ( 4 * c1.height() > 5 * mean_height() );
+      bool small = false;
+      for( int j = begin; j < characters(); ++j ) if( j != i )
+        {
+        const Character & c2 = character( j );
+        if( !c2.guesses() ) continue;
+        if( c2.maybe(' ') ) { if( isolated ) continue; else break; }
+        const int code2 = c2.guess( 0 ).code;
+        if( code2 >= 128 || !std::isalpha( code2 ) ) continue;
+        if( !capital )
+          {
+          if( 4 * c1.height() > 5 * c2.height() ) capital = true;
+          else if( std::isupper( code2 ) && code2 != 'B' && code2 != 'Q' &&
+                   ( c1.height() >= c2.height() ||
+                     Ocrad::similar( c1.height(), c2.height(), 10 ) ) )
+            capital = true;
+          else if( code2 == 't' && c1.height() >= c2.height() ) capital = true;
+
+          }
+        if( !small && std::islower( code2 ) && code2 != 'l' && code2 != 'j' )
+          {
+          if( 5 * c1.height() < 4 * c2.height() ) small = true;
+          else if( UCS::islower_small( code2 ) &&
+                   code2 != 'r' && !c2.maybe('Q')  &&
+                   ( j < i || !UCS::islower_small_ambiguous( code2 ) ) &&
+                   Ocrad::similar( c1.height(), c2.height(), 10 ) )
+            small = true;
+          }
+        }
+      if( capital && !small ) c1.insert_guess( 0, std::toupper( code ), 1 );
+      }
+    }
+  }
+
+
 void Textline::recognize2( const Charset & charset )
   {
   if( big_initials() >= characters() ) return;
@@ -137,7 +193,9 @@ void Textline::recognize2( const Charset & charset )
     }
 
   // try to separate lightly merged characters
+  // FIXME try relative minima (small pixel count surrounded by larger counts)
   // FIXME try all possible separation points
+  // FIXME try other separation paths (an irregular line, not a column)
   // FIXME sometimes leaves unconnected blobs
   for( int i = big_initials(); i < characters(); ++i )
     {
@@ -254,8 +312,9 @@ void Textline::recognize2( const Charset & charset )
     if( c1.maybe(' ') ) { begin = i + 1 ; continue; }
     if( c1.guesses() )
       {
-      int code = c1.guess(0).code;
-      if( c1.guesses() != 2 || code != 'B' || c1.guess(1).code != 'a' ) continue;
+      int code = c1.guess( 0 ).code;
+      if( c1.guesses() != 2 || code != 'B' || c1.guess( 1 ).code != 'a' )
+        continue;
       if( 4 * c1.height() > 5 * mean_height() ) continue;
       for( int j = begin; j < characters(); ++j ) if( j != i )
         {
@@ -263,7 +322,7 @@ void Textline::recognize2( const Charset & charset )
         if( c2.maybe(' ') ) break;
         if( c2.guesses() >= 1 )
           {
-          int code2 = c2.guess(0).code;
+          int code2 = c2.guess( 0 ).code;
           if( code2 >= 128 ) continue;
           if( ( std::isupper( code2 ) && code2 != 'B' && code2 != 'Q' && 5 * c1.height() < 4 * c2.height() ) ||
               ( UCS::islower_small( code2 ) && code2 != 'r' && !UCS::islower_small_ambiguous( code2 ) &&
@@ -280,9 +339,9 @@ void Textline::recognize2( const Charset & charset )
     {
     Character & c1 = character( i );
     if( c1.maybe(' ') ) { begin = i + 1 ; continue; }
-    if( c1.guesses() == 2 && c1.guess(1).code == '8' )
+    if( c1.guesses() == 2 && c1.guess( 1 ).code == '8' )
       {
-      int code = c1.guess(0).code;
+      int code = c1.guess( 0 ).code;
       if( ( code != 'a' && code != 'e' ) || 5 * c1.height() < 4 * mean_height() )
         continue;
       for( int j = begin; j < characters(); ++j ) if( j != i )
@@ -291,7 +350,7 @@ void Textline::recognize2( const Charset & charset )
         if( c2.maybe(' ') ) break;
         if( c2.guesses() >= 1 )
           {
-          int code2 = c2.guess(0).code;
+          int code2 = c2.guess( 0 ).code;
           if( code2 >= 128 ) continue;
           if( ( ( std::isalpha( code2 ) || code2 == ':' ) && 4 * c1.height() > 5 * c2.height() ) ||
               ( ( std::isdigit( code2 ) || std::isupper( code2 ) || code2 == 'l' ) &&
@@ -303,56 +362,7 @@ void Textline::recognize2( const Charset & charset )
       }
     }
 
-  // transform some small letters to capitals
-  {
-  int begin = big_initials();
-  bool isolated = false;	// isolated letters compare with all line
-  for( int i = big_initials(); i < characters(); ++i )
-    {
-    Character & c1 = character( i );
-    if( c1.maybe(' ') )
-      {
-      if( i + 2 < characters() && character( i + 2 ).maybe(' ') )
-        { begin = big_initials(); isolated = true; }
-      else { begin = i + 1; isolated = false; }
-      continue;
-      }
-    if( c1.guesses() == 1 )
-      {
-      int code = c1.guess( 0 ).code;
-      if( !UCS::islower_small_ambiguous( code ) ) continue;
-      if( 5 * c1.height() < 4 * mean_height() ) continue;
-      bool capital = ( 4 * c1.height() > 5 * mean_height() ), small = false;
-      for( int j = begin; j < characters(); ++j ) if( j != i )
-        {
-        Character & c2 = character( j );
-        if( !c2.guesses() ) continue;
-        if( c2.maybe(' ') ) { if( isolated ) continue; else break; }
-        int code2 = c2.guess( 0 ).code;
-        if( code2 >= 128 || !std::isalpha( code2 ) ) continue;
-        if( !capital )
-          {
-          if( 4 * c1.height() > 5 * c2.height() ) capital = true;
-          else if( std::isupper( code2 ) && code2 != 'B' && code2 != 'Q' &&
-                   ( c1.height() >= c2.height() ||
-                     Ocrad::similar( c1.height(), c2.height(), 10 ) ) )
-            capital = true;
-          else if( code2 == 't' && c1.height() >= c2.height() ) capital = true;
-
-          }
-        if( !small && std::islower( code2 ) && code2 != 'l' && code2 != 'j' )
-          {
-          if( 5 * c1.height() < 4 * c2.height() ) small = true;
-          else if( UCS::islower_small( code2 ) && code2 != 'r' &&
-                   ( j < i || !UCS::islower_small_ambiguous( code2 ) ) &&
-                   Ocrad::similar( c1.height(), c2.height(), 10 ) )
-            small = true;
-          }
-        }
-      if( capital && !small ) c1.insert_guess( 0, std::toupper( code ), 1 );
-      }
-    }
-  }
+  check_lower_ambiguous();
 
   // transform 'i' into 'j'
   for( int i = big_initials(); i < characters(); ++i )
@@ -430,7 +440,7 @@ void Textline::recognize2( const Charset & charset )
           if( UCS::isdigit( code2 ) )
             {
             if( Ocrad::similar( c1.height(), c2.height(), 10 ) )
-              c1.insert_guess( 0, (code == 'l') ? '1' : '0', c1.guess(0).value + 1 );
+              c1.insert_guess( 0, (code == 'l') ? '1' : '0', c1.guess( 0 ).value + 1 );
             break;
             }
           if( UCS::isalpha( code2 ) &&
@@ -446,6 +456,7 @@ void Textline::recognize2( const Charset & charset )
     Character & c1 = character( i );
     if( c1.guesses() == 1 && c1.guess( 0 ).code == 'p' )
       {
+      const int noise = std::max( 2, c1.height() / 20 );
       bool cap = false, valid_c2 = false;
       if( i < characters() - 1 && character(i+1).guesses() )
         {
@@ -457,14 +468,14 @@ void Textline::recognize2( const Charset & charset )
           switch( code )
             {
             case 'g': case 'j': case 'p': case 'q': case 'y':
-                      cap = ( c1.bottom() + 2 <= c2.bottom() ); break;
-            case 'Q': cap = ( std::abs( c1.top() - c2.top() ) <= 2 ); break;
-            default : cap = ( std::abs( c1.bottom() - c2.bottom() ) <= 2 );
+                      cap = ( c1.bottom() + noise <= c2.bottom() ); break;
+            case 'Q': cap = ( std::abs( c1.top() - c2.top() ) <= noise ); break;
+            default : cap = ( std::abs( c1.bottom() - c2.bottom() ) <= noise );
             }
           }
         }
       if( !valid_c2 && i > big_initials() && !character(i-1).maybe(' ') )
-        cap = ( std::abs( c1.bottom() - charbox(c1).bottom() ) <= 2 );
+        cap = ( std::abs( c1.bottom() - charbox(c1).bottom() ) <= noise );
       if( cap ) c1.only_guess( 'P', 0 );
       }
     }
@@ -538,7 +549,7 @@ void Textline::recognize2( const Charset & charset )
         Character & c = character( i );
         if( !c.guesses() ) break;
         bool valid = false;
-        int code = c.guess(0).code;
+        int code = c.guess( 0 ).code;
         if( ( UCS::isdigit( code ) || code == 'l' || code == 'O' || code == 'o' ) &&
             Ocrad::similar( c.height(), height, 10 ) )
           { valid = true; ++digits; }
@@ -550,11 +561,11 @@ void Textline::recognize2( const Charset & charset )
         for( i = begin; i < end; ++i )
         {
         Character & c = character( i );
-        int code = c.guess(0).code;
+        int code = c.guess( 0 ).code;
         if( code == 'l' ) code = '1';
         else if( code == 'O' || code == 'o' ) code = '0';
         else code = 0;
-        if( code ) c.insert_guess( 0, code, c.guess(0).value + 1 );
+        if( code ) c.insert_guess( 0, code, c.guess( 0 ).value + 1 );
         }
       }
     }
@@ -576,10 +587,36 @@ void Textline::recognize2( const Charset & charset )
         {
         Character & c = character( i );
         if( !character(i).maybe('I') )
-          c.insert_guess( 0, 'I', c.guess(0).value + 1 );
+          c.insert_guess( 0, 'I', c.guess( 0 ).value + 1 );
         }
     }
 
+  // choose between 'a' and 'Q'
+  for( int i = big_initials(); i < characters(); ++i )
+    {
+    Character & c = character( i );
+    if( c.guesses() == 2 &&
+        c.guess( 0 ).code == 'a' && c.guess( 1 ).code == 'Q' )
+      {
+      if( 4 * c.height() > 5 * mean_height() )
+        { c.swap_guesses( 0, 1 ); check_lower_ambiguous(); continue; }
+      if( i < characters() - 1 && character( i + 1 ).guesses() )
+        {
+        const int code = character( i + 1 ).guess( 0 ).code;
+        if( ( UCS::ishigh( code ) ) &&
+            10 * c.height() > 9 * character( i + 1 ).height() )
+          { c.swap_guesses( 0, 1 ); check_lower_ambiguous(); continue; }
+        }
+      if( i > big_initials() && character( i - 1 ).guesses() )
+        {
+        const int code = character( i - 1 ).guess( 0 ).code;
+        if( ( UCS::ishigh( code ) ) &&
+            10 * c.height() > 9 * character( i - 1 ).height() )
+          { c.swap_guesses( 0, 1 ); check_lower_ambiguous(); }
+        }
+      }
+    }
+
   // transform a vertical bar into 'l' or 'I' (or a 'l' into an 'I')
   for( int i = big_initials(); i < characters(); ++i )
     {
@@ -593,7 +630,7 @@ void Textline::recognize2( const Charset & charset )
         lcode = character( i - 1 ).guess( 0 ).code;
       if( i < characters() - 1 && character( i + 1 ).guesses() )
         rcode = character( i + 1 ).guess( 0 ).code;
-      if( UCS::isupper( rcode ) &&
+      if( ( UCS::isupper( rcode ) || UCS::isdigit( rcode ) ) &&
           ( !lcode || UCS::isupper( lcode ) || !UCS::isalnum( lcode ) ) )
         { c.insert_guess( 0, 'I', 1 ); continue; }
       if( code == 'l' ) continue;
@@ -611,6 +648,27 @@ void Textline::recognize2( const Charset & charset )
       }
     }
 
+  // transform a vertical bar into 'I' at end of word
+  for( int begin = big_initials(), end = begin; begin < characters(); begin = end + 1 )
+    {
+    end = find_space_or_hyphen( cpv, begin );
+    if( end - begin < 3 ) continue;
+    Character & ce = character( end - 1 );
+    if( !ce.maybe('|') || ce.maybe('I') ) continue;
+    const int height = ce.height();
+    int i;
+    for( i = begin; i < end - 1; ++i )
+      {
+      const Character & c = character( i );
+      if( !c.guesses() ) break;
+      const int code = c.guess( 0 ).code;
+      if( ( !UCS::isupper( code ) && !UCS::isdigit( code ) ) ||
+          !Ocrad::similar( c.height(), height, 10 ) ) break;
+      }
+    if( i >= end - 1 )
+      ce.insert_guess( 0, 'I', ce.guess( 0 ).value + 1 );
+    }
+
   // transform 'l' or '|' into UCS::SINODOT
   if( charset.enabled( Charset::iso_8859_9 ) )
   for( int i = big_initials(), begin = i; i < characters(); ++i )
@@ -683,37 +741,15 @@ void Textline::recognize2( const Charset & charset )
       }
     }
 
-  // choose between 'a' and 'Q'
-  for( int i = big_initials(), begin = i; i < characters(); ++i )
-    {
-    Character & c = character( i );
-    if( c.maybe(' ') ) { begin = i + 1 ; continue; }
-    if( c.guesses() )
-      {
-      int code = c.guess( 0 ).code;
-      if( i == begin && code == 'a' && c.guesses() == 2 && c.guess( 1 ).code == 'Q' )
-        {
-        if( 4 * c.height() > 5 * mean_height() )
-          c.swap_guesses( 0, 1 );
-        else if( i < characters() - 1 && character( i + 1 ).guesses() )
-          {
-          int code1 = character( i + 1 ).guess( 0 ).code;
-          if( ( UCS::isdigit( code1 ) || UCS::isupper( code1 ) || code1 == 'i' ) &&
-              5 * c.height() > 4 * character( i + 1 ).height() )
-          c.swap_guesses( 0, 1 );
-          }
-        }
-      }
-    }
-
   // choose between '.' and '-'
   if( characters() >= 2 )
     {
     Character & c = character( characters() - 1 );
-    if( c.guesses() >= 2 && c.guess(0).code == '.' && c.guess(1).code == '-' )
+    if( c.guesses() >= 2 &&
+        c.guess( 0 ).code == '.' && c.guess( 1 ).code == '-' )
       {
-      Character & lc = character( characters() - 2 );
-      if( lc.guesses() && UCS::isalpha( lc.guess(0).code ) )
+      const Character & lc = character( characters() - 2 );
+      if( lc.guesses() && UCS::isalpha( lc.guess( 0 ).code ) )
         c.swap_guesses( 0, 1 );
       }
     }
@@ -735,14 +771,14 @@ void Textline::recognize2( const Charset & charset )
     }
 
   // join the secuence '', '/', 'o', ' ' into a '%'
-  for( int i = big_initials(); i < characters() - 3; ++i )
+  for( int i = big_initials(); i + 2 < characters(); ++i )
     {
     Character & c1 = character( i );
     if( c1.guesses() == 1 && c1.guess( 0 ).code == UCS::DEG )
       {
       if( character( i + 1 ).maybe('/') &&
           character( i + 2 ).maybe('o') &&
-          character( i + 3 ).maybe(' ') )
+          ( i + 3 >= characters() || character( i + 3 ).maybe(' ') ) )
         {
         c1.join( character( i + 1 ) ); c1.join( character( i + 2 ) );
         delete_character( i + 2 ); delete_character( i + 1 );
