@@ -1,6 +1,6 @@
 /*  GNU Ocrad - Optical Character Recognition program
     Copyright (C) 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011,
-    2012, 2013 Antonio Diaz Diaz.
+    2012, 2013, 2014 Antonio Diaz Diaz.
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 
 #include <algorithm>
 #include <cstdio>
+#include <cstdlib>
 #include <string>
 #include <vector>
 #include <stdint.h>
@@ -110,9 +111,9 @@ inline void join_blobs( std::vector< Blob * > & blobp_vector,
 
 void ignore_abnormal_blobs( std::vector< Blob * > & blobp_vector )
   {
-  for( int i = blobp_vector.size() - 1; i >= 0; --i )
+  for( unsigned i = blobp_vector.size(); i > 0; )
     {
-    Blob & b = *blobp_vector[i];
+    Blob & b = *blobp_vector[--i];
     if( b.height() > 35 * b.width() || b.width() > 25 * b.height() )
       {
       delete blobp_vector[i];
@@ -257,7 +258,7 @@ int mean_blob_height( const std::vector< Blob * > & blobp_vector )
     if( h >= height_distrib.size() ) height_distrib.resize( h + 1 );
     ++height_distrib[h]; ++samples;
     }
-  if( !height_distrib.size() )
+  if( height_distrib.empty() )
     for( unsigned i = 0; i < blobp_vector.size(); ++i )
       {
       const unsigned h = blobp_vector[i]->height();
@@ -281,7 +282,7 @@ int mean_blob_height( const std::vector< Blob * > & blobp_vector )
 int analyse_layout( std::vector< Blob * > & blobp_vector,
                     std::vector< Zone > & zone_vector )
   {
-  if( !blobp_vector.size() ) return 0;
+  if( blobp_vector.empty() ) return 0;
   const int mean_height = mean_blob_height( blobp_vector );
   zone_vector.push_back( Zone( *blobp_vector[0] ) );
   zone_vector.back().blobp_vector.push_back( blobp_vector[0] );
@@ -316,7 +317,7 @@ int analyse_layout( std::vector< Blob * > & blobp_vector,
   blobp_vector.clear();
 
   // sort zone_vector
-  int botmax = ( zone_vector.size() ? zone_vector[0].mask.bottom() : 0 );
+  int botmax = zone_vector.empty() ? 0 : zone_vector[0].mask.bottom();
   std::vector< int > cut_index_vector;
   for( unsigned i = 1; i < zone_vector.size(); ++i )
     {
